@@ -38,6 +38,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.stream.Collectors;
 
+import htsjdk.samtools.util.*;
 import org.apache.commons.collections4.CollectionUtils;
 import org.apache.commons.lang.StringUtils;
 import org.apache.commons.math3.stat.descriptive.moment.Mean;
@@ -74,15 +75,6 @@ import htsjdk.samtools.SAMFileWriterFactory;
 import htsjdk.samtools.SAMRecord;
 import htsjdk.samtools.SAMSequenceDictionary;
 import htsjdk.samtools.SamReaderFactory;
-import htsjdk.samtools.util.CloseableIterator;
-import htsjdk.samtools.util.CloserUtil;
-import htsjdk.samtools.util.IOUtil;
-import htsjdk.samtools.util.Interval;
-import htsjdk.samtools.util.IntervalList;
-import htsjdk.samtools.util.Log;
-import htsjdk.samtools.util.PeekableIterator;
-import htsjdk.samtools.util.ProgressLogger;
-import htsjdk.samtools.util.StringUtil;
 import htsjdk.variant.variantcontext.Genotype;
 import htsjdk.variant.variantcontext.GenotypeType;
 import htsjdk.variant.variantcontext.GenotypesContext;
@@ -563,6 +555,52 @@ public class AssignCellsToSamples extends GeneFunctionCommandLineBase {
 		return processSNP(vc, sgpList, sampleNames, likelihoodCollection, addMissingValues);
 	}
 
+	/**
+	 * @param b byte value of A,C, G or T
+	 * @return very compact representation of A, C, G or T (2 bits)
+	 */
+	private static int getCompactBaseRepresentation(byte b) {
+		switch (b) {
+			case SequenceUtil.A:
+				return 0;
+			case SequenceUtil.C:
+				return 1;
+			case SequenceUtil.G:
+				return 2;
+			case SequenceUtil.T:
+				return 3;
+			default:
+				throw new IllegalArgumentException(String.format("Unexpected base %d", b));
+		}
+	}
+
+	/**
+	 *
+	 * @param a1 byte value of A,C, G or T
+	 * @param a2 byte value of A,C, G or T
+	 * @return very compact representation of this ordered pair of alleles (4 bits)
+	 */
+	private static int getAllelesToGroupBy(final byte a1, final byte a2) {
+		return (getCompactBaseRepresentation(a1) << 2) | getCompactBaseRepresentation(a2);
+	}
+
+	// Return pair of alleles in compact form for Collectors.groupingBy
+	private static int getAllelesToGroupBy(final Genotype g) {
+		return getAllelesToGroupBy(g.getAllele(0).getBases()[0], g.getAllele(1).getBases()[0]);
+	}
+
+	// We know that T,T is the largest value in the compact rep.
+	private static final int NumCompactAlleleRepresentations =
+			getAllelesToGroupBy(SequenceUtil.T, SequenceUtil.T) + 1;
+
+	// Recycle to avoid GC
+	// The index into these arrays is produced by getAllelesToGroupBy()
+	private final Genotype[] prototypeGenotypeForAllelePair = new Genotype[NumCompactAlleleRepresentations];
+	// Once a list is created, it is not destroyed but rather cleared for the next SNP.
+	// However, if prototypeGenotypeForAllelePair[i] is null, then samplesForAllelePair[i] may contain sample names
+	// from a previous iteration that are no longer relevant.
+	private final List<String>[] samplesForAllelePair = new List[NumCompactAlleleRepresentations];
+
 	/**
 	 * This updates the likelihood all cells that have one or more UMIs at this variant.
 	 * 
@@ -579,18 +617,46 @@ public class AssignCellsToSamples extends GeneFunctionCommandLineBase {
 		Iterator<Genotype> gi = gc.iterator();
 		GenotypeGQFilter gf = new GenotypeGQFilter(gi, this.GQ_THRESHOLD);
 		char refAlleleBase = StringUtil.byteToChar(vc.getReference().getBases()[0]);
-		// this would be faster if we batch updated all donors that have the same genotype.
-		// map List of <A1,A2> to list of sampleNames.
+
+		// Primitive implementation of grouping together the samples that have the same pair of alleles for this SNP.
+		// This way has many fewer objects created than groupBy.
+
+		// This array holds the first Genotype object encountered for a pair of alleles.  The index into the
+		// array is produced by getAllelesToGroupBy()
+		Arrays.fill(prototypeGenotypeForAllelePair, null);
+		// Don't bother clearing samplesForAllelePair globally.  Just clear the entries for allele pairs
+		// encountered for this SNP.
+
+		// Capture one Genotype object for each distinct pair of alleles, and get the list of
+		// sample names for each distinct pair of alleles.
+		for (final Genotype g: gf) {
+			final int compactAlleleRep = getAllelesToGroupBy(g);
+			if (prototypeGenotypeForAllelePair[compactAlleleRep] == null) {
+				prototypeGenotypeForAllelePair[compactAlleleRep] = g;
+				if (samplesForAllelePair[compactAlleleRep] == null) {
+					samplesForAllelePair[compactAlleleRep] = new ArrayList<>(sampleNames.size());
+				} else {
+					samplesForAllelePair[compactAlleleRep].clear();
+				}
+			}
+			samplesForAllelePair[compactAlleleRep].add(g.getSampleName());
+		}
+
+		// batch update all donors that have the same genotype.
+		// map allele pair to list of Genotype.
 		// then batch update likelihoods on the list of sample names.
 		// this would be a bad plan if we used genotype quality, where each sample could have a different genotype quality
 		// score.
-		for (Genotype g : gf) {
-			// use the cached genotype if available
+		for (int i = 0; i < NumCompactAlleleRepresentations; ++i) {
+			if (prototypeGenotypeForAllelePair[i] == null) {
+				continue;
+			}
+			final Genotype g = prototypeGenotypeForAllelePair[i];
 			byte[] a1Bases = g.getAllele(0).getBases();
 			byte[] a2Bases = g.getAllele(1).getBases();
 			char a1 = StringUtil.byteToChar(a1Bases[0]);
 			char a2 = StringUtil.byteToChar(a2Bases[0]);
-			String sampleName = g.getSampleName();
+			final List<String> sampleNamesForAlleles = samplesForAllelePair[i];
 			// this isn't fully implemented yet so is disabled.
 			/*
 			 * Double genotypeProbability=null; if (USE_GENOTYPE_QUALITY) { Integer genotypeQuality = g.getGQ();
@@ -598,7 +664,7 @@ public class AssignCellsToSamples extends GeneFunctionCommandLineBase {
 			 */
 			// for this sample genotype, update all the cells where we have data.
 			for (SampleGenotypeProbabilities p : sgpList) {
-				likelihoodCollection.updatelikelihoods(p, sampleName, a1, a2, null, refAlleleBase);
+				likelihoodCollection.updatelikelihoods(p, sampleNamesForAlleles, a1, a2, null, refAlleleBase);
 			}
 
 		}
