@@ -23,20 +23,12 @@
  */
 package org.broadinstitute.dropseqrna.barnyard.digitalallelecounts.sampleassignment.multisample;
 
-import java.io.File;
-import java.io.PrintStream;
-import java.text.DecimalFormat;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.Comparator;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-import java.util.stream.Collectors;
-
+import htsjdk.samtools.SAMRecord;
+import htsjdk.samtools.SAMSequenceDictionary;
+import htsjdk.samtools.SamReaderFactory;
+import htsjdk.samtools.util.*;
+import htsjdk.variant.variantcontext.VariantContext;
+import htsjdk.variant.vcf.VCFFileReader;
 import org.apache.commons.lang.StringUtils;
 import org.broadinstitute.barclay.argparser.Argument;
 import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
@@ -46,7 +38,6 @@ import org.broadinstitute.dropseqrna.barnyard.digitalallelecounts.SNPUMIBasePile
 import org.broadinstitute.dropseqrna.barnyard.digitalallelecounts.SortOrder;
 import org.broadinstitute.dropseqrna.barnyard.digitalallelecounts.sampleassignment.CellAssignmentUtils;
 import org.broadinstitute.dropseqrna.barnyard.digitalallelecounts.sampleassignment.CellCollectionSampleLikelihoodCollection;
-import org.broadinstitute.dropseqrna.barnyard.digitalallelecounts.sampleassignment.CellSampleLikelihoodCollection;
 import org.broadinstitute.dropseqrna.barnyard.digitalallelecounts.sampleassignment.SampleGenotypeProbabilities;
 import org.broadinstitute.dropseqrna.barnyard.digitalallelecounts.sampleassignment.SampleGenotypeProbabilitiesIterator;
 import org.broadinstitute.dropseqrna.cmdline.CustomCommandLineValidationHelper;
@@ -61,28 +52,20 @@ import org.broadinstitute.dropseqrna.utils.readiterators.PCRDuplicateFilteringIt
 import org.broadinstitute.dropseqrna.utils.readiterators.SamFileMergeUtil;
 import org.broadinstitute.dropseqrna.utils.readiterators.SamHeaderAndIterator;
 import org.broadinstitute.dropseqrna.vcftools.SampleAssignmentVCFUtils;
-
-import htsjdk.samtools.SAMRecord;
-import htsjdk.samtools.SAMSequenceDictionary;
-import htsjdk.samtools.SamReaderFactory;
-import htsjdk.samtools.util.CloseableIterator;
-import htsjdk.samtools.util.IOUtil;
-import htsjdk.samtools.util.Interval;
-import htsjdk.samtools.util.IntervalList;
-import htsjdk.samtools.util.Log;
-import htsjdk.samtools.util.PeekableIterator;
-import htsjdk.variant.variantcontext.VariantContext;
-import htsjdk.variant.vcf.VCFFileReader;
 import picard.cmdline.StandardOptionDefinitions;
-import picard.util.TabbedTextFileWithHeaderParser;
-import picard.util.TabbedTextFileWithHeaderParser.Row;
+
+import java.io.File;
+import java.io.PrintStream;
+import java.text.DecimalFormat;
+import java.util.*;
+import java.util.stream.Collectors;
 
 @CommandLineProgramProperties(summary = "Detect Doublets in Dropulation Data.  Uses the outputs of AssignCellsToSamples to make decisions.  It's highly recommended to use the VCF output from AssignCellsToSamples as input, as"
 		+ "the memory usage of a full VCF may be prohibitive compared to the AssignCellsToSamples VCF, which contains only the variants that were observed in the data.  This also greatly speeds up"
 		+ "analysis.", oneLineSummary = "Detect Doublets in Dropulation Data", programGroup = DropSeq.class)
 public class DetectDoublets extends GeneFunctionCommandLineBase {
 
-	private static final Log log = Log.getInstance(DetectDoublets.class);
+	private static final Log log = Log.getInstance(DetectDoublets.class); 
 
 	@Argument(shortName = StandardOptionDefinitions.INPUT_SHORT_NAME, doc = "The input SAM or BAM file to analyze. This argument can accept wildcards, or a file with the suffix .bam_list that contains the locations of multiple BAM files", minElements = 1)
 	public List<File> INPUT_BAM;
@@ -124,7 +107,10 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 			+ "ignore GQ values if they are not set in the genotype info field.  If the GQ field is not set in the VCF header, this will be set to -1 by default.")
 	public Integer GQ_THRESHOLD = 30;
 	
-	@Argument(doc = "A file with a list of samples in the VCF to consider as samples in the doublets.  This subsets the VCF into a smaller data set containing only the samples listed. The file has 1 column with no header.", optional = false)
+	@Argument(doc = "A file with a list of samples in the VCF to consider as samples in the doublets.  This subsets the VCF into a smaller data set containing only the samples listed. "
+			+ "The file has 1 column with no header.  If this list contains only one donor and no contaminating donors were found via single donor assignment, "
+			+ "doublet detection calculations will not take place.  Instead, the program will emit a default output for each cell, and the program "
+			+ "will then quit with an exit status.", optional = false)
 	public File SAMPLE_FILE;
 
 	@Argument(doc = "Instead of useing base qualities to determine error rate, use a fixed error rate instead. This is rounded to the nearest phread score internally.", optional = true)
@@ -209,7 +195,7 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 			perDonorWriter = new ErrorCheckingPrintStream(IOUtil.openFileForWriting(this.OUTPUT_ALL_PAIRS));
 			writeAssignmentHeader(perDonorWriter, pairsToTest, false, true);
 		}
-
+	
 		PrintStream perSNPWriter = null;
 		if (OUTPUT_PER_SNP != null) {
 			IOUtil.assertFileIsWritable(this.OUTPUT_PER_SNP);
@@ -258,8 +244,13 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 
 		// Keep all the barcodes that are in the barcode list AND in the single donor assignments.
 		List<String> cellBarcodes = getCellBarcodes(this.CELL_BC_FILE, bestDonorForCell, this.TEST_UNEXPECTED_DONORS);
-		;
-
+		
+		// If there is only one donor at this point, doublet detection should not continue.  
+		if (allDonorsList.size()<2) {
+			singleDonorGracefulExit(bestDonorForCell, writer, perDonorWriter, perSNPWriter);
+			return 0;
+		}
+		
 		// Pass a list of all donors requested + best calls if TEST_UNEXPECTED_DONORS is true.
 		// set the %passing to be 0, since the snpIntervals will properly filter on the right set of donors, but you want a
 		// super-set of donors available
@@ -305,13 +296,18 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 				FindOptimalDonorMixture fodm = new FindOptimalDonorMixture(f);
 
 				// AllPairedSampleAssignmentsForCell allAssignments = fodm.findBestDonorPair(bestDonor, donorList, FORCED_RATIO);
-				List<String> donorsThisCell = getExpectedSecondDonorsRankedByLikelihood(cell, cslc, pairsToTest, donorList, bestDonor);
-				AllPairedSampleAssignmentsForCell allAssignments = fodm.findBestDonorPair(bestDonor, donorsThisCell, FORCED_RATIO, SCALE_LIKELIHOODS);
+				// List<String> donorsThisCell = getExpectedSecondDonorsRankedByLikelihood(cell, cslc, pairsToTest, donorList, bestDonor);
+				AllPairedSampleAssignmentsForCell allAssignments = fodm.findBestDonorPair(bestDonor, donorList, FORCED_RATIO, SCALE_LIKELIHOODS);
 				SamplePairAssignmentForCell best = allAssignments.getBestAssignment();
-				/*
-				 * if (best==null) { log.info("No best pair found for cell "+ cell); continue; }
-				 */
-				double bestPairPvalue = allAssignments.getBestPairPvalue();
+
+				// edge case: assignment is null because there's no data for this cell.  This only happens
+				// when a user has a cell selection error or similar and attempts to call cell barcodes that aren't cells.				
+				if (best==null) { 
+					log.warn("No best pair found for cell ["+cell+"] due to no informative UMIs.  Cell selection or similar problem?");
+					best= SamplePairAssignmentForCell.constructEmptyResult(cell, bestDonorForCell.get(cell));
+				}
+				 
+				double bestPairPvalue = allAssignments.getBestPairPvalue();																	
 				writeAssignment(best, bestPairPvalue, writer, false);
 				if (OUTPUT_ALL_PAIRS != null) {
 					writeAssignment(allAssignments.getBestAssignment(), null, perDonorWriter, true);
@@ -336,6 +332,40 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 		log.info("Finished!");
 		return 0;
 	}
+	
+	
+	/**
+	 * In the strange edge case where there is only a single donor to be tested, write out a default output file instead of going through testing.
+	 * This function additionally closes all potentially open writers and runs logging.
+	 * @param bestDonorForCell A map containing cell barcodes and the best donor for each cell.
+	 * @param writer The file to write to per-cell outputs.	 
+	 * @param perDonorWriter Closes this file if not null.
+	 * @param perSNPWriter Closes this file if not null.
+	 */
+	void singleDonorGracefulExit(Map<String, String> bestDonorForCell, PrintStream writer, 
+			PrintStream perDonorWriter, PrintStream perSNPWriter) {
+		// clean up more detailed file writers. 
+		if (OUTPUT_ALL_PAIRS!=null) perDonorWriter.close();
+		if (OUTPUT_PER_SNP != null) perSNPWriter.close();
+		// write a default output per cell close results and quit.			
+		log.error("The donor file only contained a single donor, and no additional donors were detected by single donor assignment.  Doublet detection will not continue.  "
+				+ "A default output will be written to perserve downstream pipeline functionality.");
+		writeSingleDonorEdgeCaseOutput(bestDonorForCell, writer);				
+	}
+	
+	/**
+	 * In the strange edge case where there is only a single donor to be tested, write out a default output file instead of going through testing.
+	 * @param bestDonorForCell A map containing cell barcodes and the best donor for each cell.
+	 * @param writer The file to write to
+	 */
+	void writeSingleDonorEdgeCaseOutput(Map<String, String> bestDonorForCell, PrintStream writer) {
+		for (String cell: bestDonorForCell.keySet()) {
+			SamplePairAssignmentForCell best = SamplePairAssignmentForCell.constructEmptyResult(cell, bestDonorForCell.get(cell));
+			writeAssignment(best, 1.0E-101d, writer, false);
+		}
+		writer.close();
+		
+	}
 
 	/**
 	 * Filter the map of cell barcode -> donor to only retain cell barcodes where the assigned donor is in the donor list.
@@ -344,7 +374,7 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 	 * @param donorList
 	 * @return A submap of the input map where all values are contained in the donor list.
 	 */
-	private Map<String, String> filterDonorMap(Map<String, String> bestDonorForCell, List<String> donorList) {
+	Map<String, String> filterDonorMap(Map<String, String> bestDonorForCell, List<String> donorList) {
 		Set<String> dl = new HashSet<String>(donorList);
 		Map<String, String> result = new HashMap<String, String>();
 
@@ -442,11 +472,7 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 	}
 
 	public static void writeAssignment(final SamplePairAssignmentForCell assignment, final Double bestPairPvalue, final PrintStream out,
-			final boolean writeScaledLikelihood) {
-		// cell sampleOneMixtureRatio sampleOne sampleOneLikelihood
-		// sampleTwo sampleTwoLikelihood mixedSample mixedSampleLikelihood
-		// num_paired_snps num_inform_snps lr_test_stat sampleOneWrongAlleleCount
-		// sampleTwoWrongAlleleCount ratio bestLikelihood bestSample
+			final boolean writeScaledLikelihood) {				
 		String mixture = mixtureFormat.format(assignment.getMixture());
 
 		List<String> line = new ArrayList<String>(
@@ -461,49 +487,14 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 
 		if (bestPairPvalue != null)
 			line.add(bestPairPvalue.toString());
+		
 		if (writeScaledLikelihood)
 			line.add(Double.toString(assignment.getScaledBestLikelihood()));
 
 		String h = StringUtils.join(line, "\t");
 		out.println(h);
 	}
-
-	private static SamplePairAssignmentForCell parseAssignment(Row row) {
-		String cellBarcode = row.getField("cell");
-		double mixtureRatio = Double.parseDouble(row.getField("sampleOneMixtureRatio"));
-		String sampleOne = row.getField("sampleOne");
-		double sampleOneSingleLikelihood = Double.parseDouble(row.getField("sampleOneLikelihood"));
-		String sampleTwo = row.getField("sampleTwo");
-		double sampleTwoSingleLikelihood = Double.parseDouble(row.getField("sampleTwoLikelihood"));
-		double doubletLikelihood = Double.parseDouble(row.getField("mixedSampleLikelihood"));
-		int numSNPs = row.getIntegerField("num_paired_snps");
-		int numInformativeSNPs = row.getIntegerField("num_inform_snps");
-		int numUMIs = row.getIntegerField("num_umi");
-		int numInformativeUMIs = row.getIntegerField("num_inform_umis");
-		int impossibleAllelesSampleOne = row.getIntegerField("sampleOneWrongAlleleCount");
-		int impossibleAllelesSampleTwo = row.getIntegerField("sampleTwoWrongAlleleCount");
-		int numInformativeHomozygousUMIsSampleOne = row.getIntegerField("num_homozygous_inform_umis_s1");
-		int numInformativeHomozygousUMIsSampleTwo = row.getIntegerField("num_homozygous_inform_umis_s2");
-
-		return new SamplePairAssignmentForCell(cellBarcode, sampleOne, sampleTwo, sampleOneSingleLikelihood, sampleTwoSingleLikelihood, doubletLikelihood,
-				mixtureRatio, impossibleAllelesSampleOne, impossibleAllelesSampleTwo, numInformativeSNPs, numSNPs, numUMIs, numInformativeUMIs,
-				numInformativeHomozygousUMIsSampleOne, numInformativeHomozygousUMIsSampleTwo);
-	}
-
-	public static List<SamplePairAssignmentForCell> parseAssignmentsFile(File assignmentsFile) {
-		TabbedTextFileWithHeaderParser parser = new TabbedTextFileWithHeaderParser(assignmentsFile);
-		CloseableIterator<TabbedTextFileWithHeaderParser.Row> rowIter = parser.iterator();
-
-		List<SamplePairAssignmentForCell> assignmentsList = new ArrayList<>();
-		while (rowIter.hasNext()) {
-			Row row = rowIter.next();
-			assignmentsList.add(parseAssignment(row));
-		}
-		rowIter.close();
-		parser.close();
-		return assignmentsList;
-	}
-
+	
 	public PeekableIterator<List<SampleGenotypeProbabilities>> prepareIterator(final IntervalList snpIntervals, List<String> cellBarcodes) {
 
 		SamReaderFactory factory = SamReaderFactory.makeDefault().enable(SamReaderFactory.Option.EAGERLY_DECODE);
@@ -543,7 +534,7 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 		return (peekableIter);
 	}
 
-	private List<String> getCellBarcodes(File cellBarcodeFile, Map<String, String> bestDonorForCell, boolean testUnexpectedDonors) {
+	List<String> getCellBarcodes(File cellBarcodeFile, Map<String, String> bestDonorForCell, boolean testUnexpectedDonors) {
 		List<String> cellBarcodes = ParseBarcodeFile.readCellBarcodeFile(this.CELL_BC_FILE);
 		int numBarcodes = cellBarcodes.size();
 
@@ -590,35 +581,7 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 
 	}
 
-	private List<String> getExpectedSecondDonorsRankedByLikelihood(String cellBarcode, CellCollectionSampleLikelihoodCollection cslc, int pairsToTest,
-			List<String> expectedDonors, String bestDonor) {
-		// short circuit if the requested number of donors is the same as the number of expected donors IE the non-optimized
-		// strategy.
-		if (expectedDonors.size() == pairsToTest)
-			return expectedDonors;
-
-		CellSampleLikelihoodCollection c = cslc.getLikelihoodCollection(cellBarcode);
-		if (c == null)
-			throw new IllegalArgumentException("Could not find single donor likelihoods for cell " + cellBarcode);
-		List<String> rankedDonors = c.getDonorsRankedByAssignmentLikelihood();
-		// exclude the best donor from the ranked list. We don't want to select that.
-		rankedDonors.remove(bestDonor);
-
-		Set<String> expected = new HashSet<String>(expectedDonors);
-		// filter ranked donors by expected.
-		List<String> rankedExpectedDonors = rankedDonors.stream().filter(x -> expected.contains(x)).collect(Collectors.toList());
-
-		// if pairs to test is set too high somehow (the sample lists doesn't match up with the input single donor assignments)
-		// then limit the return.
-		if (rankedExpectedDonors.size() < pairsToTest) {
-			pairsToTest = rankedExpectedDonors.size();
-		}
-		// get the top <X> donors.
-		rankedExpectedDonors = rankedExpectedDonors.subList(0, pairsToTest);
-		return rankedExpectedDonors;
-
-	}
-
+		
 	@Override
 	protected String[] customCommandLineValidation() {
 		IOUtil.assertFileIsReadable(this.VCF);
@@ -658,5 +621,37 @@ public class DetectDoublets extends GeneFunctionCommandLineBase {
 	public static void main(final String[] args) {
 		System.exit(new DetectDoublets().instanceMain(args));
 	}
+	
+	/*
+	private List<String> getExpectedSecondDonorsRankedByLikelihood(String cellBarcode, CellCollectionSampleLikelihoodCollection cslc, int pairsToTest,
+			List<String> expectedDonors, String bestDonor) {
+		// short circuit if the requested number of donors is the same as the number of expected donors IE the non-optimized
+		// strategy.
+		if (expectedDonors.size() == pairsToTest)
+			return expectedDonors;
+		
+		CellSampleLikelihoodCollection c = cslc.getLikelihoodCollection(cellBarcode);
+		if (c == null)
+			throw new IllegalArgumentException("Could not find single donor likelihoods for cell " + cellBarcode);
+		List<String> rankedDonors = c.getDonorsRankedByAssignmentLikelihood();
+		// exclude the best donor from the ranked list. We don't want to select that.
+		rankedDonors.remove(bestDonor);
+
+		Set<String> expected = new HashSet<String>(expectedDonors);
+		// filter ranked donors by expected.
+		List<String> rankedExpectedDonors = rankedDonors.stream().filter(x -> expected.contains(x)).collect(Collectors.toList());
+
+		// if pairs to test is set too high somehow (the sample lists doesn't match up with the input single donor assignments)
+		// then limit the return.
+		if (rankedExpectedDonors.size() < pairsToTest) {
+			pairsToTest = rankedExpectedDonors.size();
+		}
+		// get the top <X> donors.
+		rankedExpectedDonors = rankedExpectedDonors.subList(0, pairsToTest);
+		return rankedExpectedDonors;
+
+	}
+	*/
+
 
 }
