@@ -45,6 +45,9 @@ import java.util.List;
         programGroup = DropSeq.class)
 public class TagBamWithReadSequenceExtended extends CommandLineProgram {
 
+	public static final String NUM_FAILED_BASES_COLUMN = "num_failed_bases";
+	public static final String NUM_BARCODES_COLUMN = "num_barcodes";
+
 	private final Log log = Log.getInstance(TagBamWithReadSequenceExtended.class);
 
 	@Argument(shortName = StandardOptionDefinitions.INPUT_SHORT_NAME, doc = "The input SAM or BAM file to analyze.")
@@ -56,7 +59,7 @@ public class TagBamWithReadSequenceExtended extends CommandLineProgram {
 	@Argument(doc = "Summary of barcode base quality",optional=true)
 	public File SUMMARY;
 
-	@Argument(doc="Base range to extract, seperated by a dash.  IE: 1-4.  Can extract multiple ranges by seperating them by a colon.  For example 1-4:17-22 extracts the first 4 bases, then the 17-22 bases, and glues the sequence together into a single sequence for a tag.")
+	@Argument(doc="Base range to extract, seperated by a dash.  E.g 1-4.  Can extract multiple ranges by separating them by a colon.  For example 1-4:17-22 extracts the first 4 bases, then the 17-22 bases, and glues the sequence together into a single sequence for a tag.")
 	public String BASE_RANGE;
 
 	@Argument(doc = "The sequence can be from the first or second read [1/2].  ")
@@ -246,9 +249,9 @@ public class TagBamWithReadSequenceExtended extends CommandLineProgram {
 	}
 
 
-	private void writeOutput (final FailedBaseMetric result, final File output) {
+	public static void writeOutput (final FailedBaseMetric result, final File output) {
 		BufferedWriter writer = OutputWriterUtil.getWriter(output);
-		String [] header = {"num_failed_bases", "num_barcodes"};
+		String [] header = {NUM_FAILED_BASES_COLUMN, NUM_BARCODES_COLUMN};
 		String h = StringUtils.join(header, "\t");
 		OutputWriterUtil.writeResult(h, writer);
 		for (int i=0; i<result.getLength(); i++) {
