@@ -26,8 +26,11 @@ package org.broadinstitute.dropseqrna.barnyard.digitalexpression.tools;
 import java.io.File;
 import java.io.IOException;
 import java.util.Arrays;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Set;
 
+import org.broadinstitute.dropseqrna.utils.TestUtils;
 import org.junit.Assert;
 import org.testng.annotations.Test;
 
@@ -43,6 +46,8 @@ public class DGEMatrixTest {
 	private final File matrixMarketMatrixFile = new File ("testdata/org/broadinstitute/transcriptome/barnyard/digitalexpression/tenXMatrixMarket.mtx");
 	private final File matrixMarketCellFile = new File ("testdata/org/broadinstitute/transcriptome/barnyard/digitalexpression/tenXMatrixMarketCellBarcodes.tsv");
 	private final File matrixMarketGeneFile = new File ("testdata/org/broadinstitute/transcriptome/barnyard/digitalexpression/tenXMatrixMarketGenes.tsv");
+	// same cells as exampleOne, but different genes
+	private final File metageneExampleOne = new File("testdata/org/broadinstitute/transcriptome/barnyard/digitalexpression/metagene_dge_example1.txt.gz");
 
 
 
@@ -235,6 +240,7 @@ public class DGEMatrixTest {
 
 	@Test
 	public void testRemoveCells() {
+		//TODO: this test kinda sucks.  Repeated values can lead to mistakes in this test that are not detected.
 		final DGEMatrix result= DGEMatrix.parseFile(exampleOne);
 		final String [] cellsToRemove = {"CACTAAAGCCAG", "ATGGTCTCAAAC"};
 		result.removeCellBarcodes(Arrays.asList(cellsToRemove));
@@ -400,4 +406,37 @@ public class DGEMatrixTest {
     }
 
 
+	/**
+	 * Merge 2 DGEs with the same cell barcodes, but set of genes disjoint.
+	 */
+	@Test
+	public void testMergeExpressionForCells() {
+		final DGEMatrix m1 = DGEMatrix.parseFile(exampleOne);
+		final DGEMatrix m2 = DGEMatrix.parseFile(metageneExampleOne);
+		final DGEMatrix merged = m1.merge(m2);
+		final DGEMatrix mergedOtherWay = m2.merge(m1);
+		Assert.assertEquals(merged.getMatrix(), mergedOtherWay.getMatrix());
+		final Set<String> allCellBarcodes = new HashSet<>(m1.getCellBarcodes());
+		allCellBarcodes.addAll(m2.getCellBarcodes());
+		Assert.assertEquals(allCellBarcodes, new HashSet<>(merged.getCellBarcodes()));
+		final Set<String> allGenes = new HashSet<>(m1.getGenes());
+		allGenes.addAll(m2.getGenes());
+		Assert.assertEquals(allGenes, new HashSet<>(merged.getGenes()));
+		Assert.assertTrue(TestUtils.testSorted(merged.getGenes()));
+		for (final String gene : merged.getGenes()) {
+			final DGEMatrix whichMatrix;
+			if (m1.getGenes().contains(gene)) {
+				whichMatrix = m1;
+			} else {
+				whichMatrix = m2;
+			}
+			for (final String cellBarcode : merged.getCellBarcodes()) {
+				if (whichMatrix.getCellBarcodes().contains(cellBarcode)) {
+					Assert.assertEquals(whichMatrix.getExpression(gene, cellBarcode), merged.getExpression(gene, cellBarcode), 0.0);
+				} else {
+					Assert.assertEquals(0.0, merged.getExpression(gene, cellBarcode), 0.0);
+				}
+			}
+		}
+	}
 }
