@@ -23,9 +23,9 @@
  */
 package org.broadinstitute.dropseqrna.utils;
 
+import htsjdk.samtools.metrics.MetricsFile;
 import htsjdk.samtools.util.IOUtil;
 import htsjdk.samtools.util.StringUtil;
-import org.apache.commons.io.FileUtils;
 import org.broadinstitute.dropseqrna.utils.io.ErrorCheckingPrintStream;
 import org.junit.Assert;
 import org.testng.Reporter;
@@ -47,7 +47,9 @@ public class SplitBamByCellTest {
 
     public static final File TEST_BAM = new File("testdata/org/broadinstitute/dropseq/barnyard/DgeStrandFuncTest/DgeStrandFuncTest.bam");
     private static final File EXPECTED_REPORT = new File ("testdata/org/broadinstitute/dropseq/utils/SplitBamByCell.report");
+    private static final File EXPECTED_MANIFEST = new File ("testdata/org/broadinstitute/dropseq/utils/SplitBamByCell.split_bam_manifest");
     private static final File QUERYNAME_SORTED_BAM = new File("testdata/org/broadinstitute/dropseq/metrics/compute_umi_sharing.unmapped.sam");
+    private static final String GENE_FUNCTION_TAG = "gf";
 
     @Test
 	public void testDoWork() {
@@ -66,6 +68,7 @@ public class SplitBamByCellTest {
         bamSplitter.DELETE_INPUTS = false;
         bamSplitter.OVERWRITE_EXISTING = true;
         bamSplitter.REPORT = report;
+        bamSplitter.OUTPUT_MANIFEST = TestUtils.getTempReportFile("SplitBamByCell.", ".split_bam_manifest");
         TestUtils.setInflaterDeflaterIfMacOs();
 		Assert.assertEquals(bamSplitter.doWork(), 0);
 
@@ -111,12 +114,23 @@ public class SplitBamByCellTest {
             tagsComparator.TAGS = new ArrayList<>(Arrays.asList("XC", "XM"));
             Assert.assertEquals(tagsComparator.doWork(), 0);
 
-			Assert.assertTrue(FileUtils.contentEquals(report, EXPECTED_REPORT));
+            Assert.assertTrue(TestUtils.testMetricsFilesEqual(report, EXPECTED_REPORT));
+
+            // Manifest content will not be identical because of temp file names, so just compare fields expected
+            // to be consistent.
+            final Map<String, Integer> expectedManifest = loadCellBarcodeManifest(EXPECTED_MANIFEST);
+            final Map<String, Integer> actualManifest = loadCellBarcodeManifest(bamSplitter.OUTPUT_MANIFEST);
+            Assert.assertEquals(expectedManifest, actualManifest);
 		} catch (IOException ex) {
 			throw new RuntimeException("Error running the test", ex);
 		}
     }
 
+    private Map<String, Integer> loadCellBarcodeManifest(final File manifest) {
+        final List<CellBarcodeSplitBamMetric> beans = MetricsFile.readBeans(manifest);
+        return(beans.stream().collect(Collectors.toMap(b -> b.CELL_BARCODE, b -> b.SPLIT_BAM_INDEX)));
+    }
+
     @Test
     public void testTargetSplitSizeAndOverwrite() throws IOException {
         SplitBamByCell bamSplitter;
@@ -234,6 +248,119 @@ public class SplitBamByCellTest {
         TestUtils.assertSamRecordsSame(mergedOutputBAM, QUERYNAME_SORTED_BAM);
     }
 
+    @Test
+    public void testSplitGeneThresholdDefault() {
+        final SplitBamByCell bamSplitter = new SplitBamByCell();
+
+        final File mergedOutputBAM = TestUtils.getTempReportFile("SplitBamByCell.merged", ".sam");
+        final File filterTestBAM = TestUtils.getTempReportFile("SplitBamByCell.filtered", ".sam");
+        mergedOutputBAM.deleteOnExit();
+        filterTestBAM.deleteOnExit();
+
+        bamSplitter.INPUT = Collections.singletonList(TEST_BAM);
+        bamSplitter.OUTPUT =
+                TestUtils.getTempReportFile("SplitBamByCell.", "." + bamSplitter.OUTPUT_SLUG + ".bam");
+        bamSplitter.OUTPUT_LIST = TestUtils.getTempReportFile("SplitBamByCell.", ".list");
+        bamSplitter.NUM_OUTPUTS = 3;
+        bamSplitter.DELETE_INPUTS = false;
+        bamSplitter.OVERWRITE_EXISTING = true;
+        bamSplitter.REPORT = TestUtils.getTempReportFile("SplitBamByCell.", ".report");
+        bamSplitter.SPLIT_TAG = GENE_FUNCTION_TAG;
+        TestUtils.setInflaterDeflaterIfMacOs();
+        Assert.assertEquals(bamSplitter.doWork(), 0);
+
+        final List<File> splitBAMFileList = FileListParsingUtils.readFileList(bamSplitter.OUTPUT_LIST);
+        for (final File f : splitBAMFileList) {
+            f.deleteOnExit();
+        }
+
+        // Merge the split BAM files
+        final MergeSamFiles fileMerger = new MergeSamFiles();
+        final List<String> fileMergerArgs = new ArrayList<>();
+        for (final File splitBAMFilePath : splitBAMFileList) {
+            fileMergerArgs.add("INPUT=" + splitBAMFilePath.getAbsolutePath());
+        }
+        fileMergerArgs.add("OUTPUT=" + mergedOutputBAM.getAbsolutePath());
+        fileMergerArgs.add("USE_JDK_DEFLATER=" + TestUtils.isMacOs());
+        fileMergerArgs.add("SORT_ORDER=coordinate");
+
+        Assert.assertEquals(fileMerger.instanceMain(fileMergerArgs.toArray(new String[0])), 0);
+
+        // Filter the original BAM file (Uses a little bit of code+cpu here versus checking in megabytes of data to git)
+        final FilterBam fileFilter = new FilterBam();
+        final List<String> fileFilterArgs = new ArrayList<>();
+        fileFilterArgs.add("INPUT=" + TEST_BAM.getAbsolutePath());
+        fileFilterArgs.add("OUTPUT=" + filterTestBAM.getAbsolutePath());
+        fileFilterArgs.add("USE_JDK_DEFLATER=" + TestUtils.isMacOs());
+        fileFilterArgs.add("TAG_RETAIN=" + GENE_FUNCTION_TAG);
+
+        Assert.assertEquals(fileFilter.instanceMain(fileFilterArgs.toArray(new String[0])), 0);
+
+        TestUtils.assertSamRecordsSame(mergedOutputBAM, filterTestBAM);
+    }
+
+    @Test
+    public void testSplitGeneThresholdBadTag() {
+        final SplitBamByCell bamSplitter = new SplitBamByCell();
+
+        final File mergedOutputBAM = TestUtils.getTempReportFile("SplitBamByCell.merged.", ".sam");
+        mergedOutputBAM.deleteOnExit();
+
+        bamSplitter.INPUT = Collections.singletonList(TEST_BAM);
+        bamSplitter.OUTPUT =
+                TestUtils.getTempReportFile("SplitBamByCell.", "." + bamSplitter.OUTPUT_SLUG + ".bam");
+        bamSplitter.OUTPUT_LIST = TestUtils.getTempReportFile("SplitBamByCell.", ".list");
+        bamSplitter.NUM_OUTPUTS = 3;
+        bamSplitter.DELETE_INPUTS = false;
+        bamSplitter.OVERWRITE_EXISTING = true;
+        bamSplitter.REPORT = TestUtils.getTempReportFile("SplitBamByCell.", ".report");
+        bamSplitter.SPLIT_TAG = "BD"; // any tag that isn't present in the test BAM
+        TestUtils.setInflaterDeflaterIfMacOs();
+        boolean exceptionThrown = false;
+        try {
+            bamSplitter.doWork();
+        } catch (final IllegalArgumentException e) {
+            exceptionThrown = true;
+        }
+        Assert.assertTrue(exceptionThrown);
+    }
+
+    @Test(dataProvider = "testSplitGeneThresholdFailDataProvider")
+    public void testSplitGeneThresholdFail(final Double passingReadThreshold) {
+        final SplitBamByCell bamSplitter = new SplitBamByCell();
+
+        final File mergedOutputBAM = TestUtils.getTempReportFile("SplitBamByCell.merged.", ".sam");
+        mergedOutputBAM.deleteOnExit();
+
+        bamSplitter.INPUT = Collections.singletonList(TEST_BAM);
+        bamSplitter.OUTPUT =
+                TestUtils.getTempReportFile("SplitBamByCell.", "." + bamSplitter.OUTPUT_SLUG + ".bam");
+        bamSplitter.OUTPUT_LIST = TestUtils.getTempReportFile("SplitBamByCell.", ".list");
+        bamSplitter.NUM_OUTPUTS = 3;
+        bamSplitter.DELETE_INPUTS = false;
+        bamSplitter.OVERWRITE_EXISTING = true;
+        bamSplitter.REPORT = TestUtils.getTempReportFile("SplitBamByCell.", ".report");
+        bamSplitter.SPLIT_TAG = GENE_FUNCTION_TAG;
+        bamSplitter.PASSING_READ_THRESHOLD = passingReadThreshold;
+        TestUtils.setInflaterDeflaterIfMacOs();
+        boolean exceptionThrown = false;
+        try {
+            bamSplitter.doWork();
+        } catch (final IllegalArgumentException e) {
+            exceptionThrown = true;
+        }
+        Assert.assertTrue(exceptionThrown);
+    }
+
+    @DataProvider
+    public Object[][] testSplitGeneThresholdFailDataProvider() {
+        return new Object[][] {
+                {-1d},
+                {0.9},
+                {60000d},
+        };
+    }
+
     @Test(dataProvider = "testDeleteInputsAndDefaultOutputsDataProvider")
     public void testDeleteInputsAndDefaultOutputs(final Boolean DELETE_INPUT_INDICES, final boolean createIndex) throws IOException {
         final File tempDir = Files.createTempDirectory("SplitBamByCellTest.").toFile();
@@ -244,12 +371,12 @@ public class SplitBamByCellTest {
         final File index;
         if (createIndex) {
             index = createBamIndexPath(inputFile);
-            index.createNewFile();
+            Assert.assertTrue(index.createNewFile());
             index.deleteOnExit();
         } else {
             index = null;
         }
-        final File symlink1 = new File(tempDir, "symlink1" + SplitBamByCell.BAM_EXTENSION);
+        final File symlink1 = new File(tempDir, "symlink1" + AbstractSplitBamClp.BAM_EXTENSION);
         Files.createSymbolicLink(symlink1.toPath(), inputFile.toPath());
         symlink1.deleteOnExit();
         final File indexSymlink1;
@@ -261,7 +388,7 @@ public class SplitBamByCellTest {
             indexSymlink1 = null;
         }
         final String symlinkBasename = "symlink";
-        final File inputSymlink = new File(tempDir, symlinkBasename + SplitBamByCell.BAM_EXTENSION);
+        final File inputSymlink = new File(tempDir, symlinkBasename + AbstractSplitBamClp.BAM_EXTENSION);
         Files.createSymbolicLink(inputSymlink.toPath(), symlink1.toPath());
         inputSymlink.deleteOnExit();
         if (createIndex) {
@@ -286,9 +413,9 @@ public class SplitBamByCellTest {
         final Set<String> outputFileNames = outputFiles.stream().map(File::getName).collect(Collectors.toSet());
         Reporter.log(String.format("Files found: %s", StringUtil.join(", ", outputFileNames)), true);
 
-        final List<String> expectedExtensions = new ArrayList<>(Arrays.asList(SplitBamByCell.BAM_LIST_EXTENSION, SplitBamByCell.BAM_REPORT_EXTENSION));
+        final List<String> expectedExtensions = new ArrayList<>(Arrays.asList(AbstractSplitBamClp.BAM_LIST_EXTENSION, AbstractSplitBamClp.BAM_REPORT_EXTENSION));
         for (int i = 0; i < NUM_OUTPUTS; ++i) {
-            expectedExtensions.add("." + i + SplitBamByCell.BAM_EXTENSION);
+            expectedExtensions.add("." + i + AbstractSplitBamClp.BAM_EXTENSION);
         }
         // split BAMs + bam_list and report
         Assert.assertEquals(expectedExtensions.size(), outputFiles.size());
